`include "cache.h"
`include "memory.h"
`include "stdbus.h"


module Cache (
        // Asynchronous Driver Bus
        DStrobe,
        DRW,
        DAddress,
        DData,
        DReady,

        // Synchronous Memory Bus
        MStrobe,
        MRW,
        MAddress,
        MData,

        Reset,
        Clk );

input           DStrobe;
input           DRW;
input   [`ADDR] DAddress;
inout   [`DATA] DData;
output          DReady;

output          MStrobe;
output          MRW;
output  [`ADDR] MAddress;
inout   [`DATA] MData;

input           Reset;
input           Clk;

// Bidirectional buses
wire            DDataOE;        // Driver Data Output Enable
wire            MDataOE;        // Memory Data Output Enable
wire    [`DATA] DDataOut;
wire	[`DATA]	DData;
wire	[`DATA] Mdata;
wire    [`ADDR] MAddress = DAddress;
wire    [`TAG]  TagRamTag;

TagRam TagRam (
        .Address        (DAddress[`INDEX]),
        .TagIn          (DAddress[`TAG]),
        .TagOut         (TagRamTag[`TAG]),
        .Write          (Write),
        .Clk            (Clk) );

ValidRam ValidRam (
        .Address        (DAddress[`INDEX]),
        .ValidIn        (1'b1),
        .ValidOut       (Valid),
        .Write          (Write),
        .Reset          (Reset),
        .Clk            (Clk) );

wire    [`DATA] DataRamDataOut;
wire    [`DATA] DataRamDataIn;


DataMux DDataMux (
        .S      (DDataSelect),
        .A      (MData),
        .B      (DataRamDataOut),
        .Z      (DDataOut) );
        
TriStateBuffer MDataBuffer (
         .in	(DData),
         .out	( MData),
         .enable(MDataOE) );
         
TriStateBuffer DDataBuffer (
         .in	(DDataOut),
         .out	(DData),
         .enable(DDataOE) );

DataRam DataRam (
        .Address        (DAddress[`INDEX]),
        .DataIn         (DData),
        .DataOut        (DataRamDataOut),
        .Write          (Write),
        .Clk            (Clk) );

Comparator Comparator (
        .Tag1   (DAddress[`TAG]),
        .Tag2   (TagRamTag),
        .Match  (Match) );

CacheControl CacheControl (
        .DStrobe        (DStrobe),
        .DRW            (DRW),
        .DReady         (DReady),
        .Match          (Match),
        .Valid          (Valid),
        .Write          (Write),
        .DDataSelect    (DDataSelect),
        .MDataOE        (MDataOE),
        .DDataOE        (DDataOE),
        .MStrobe        (MStrobe),
        .MRW            (MRW),
        .Reset          (Reset),
        .Clk            (Clk) );

endmodule
