//
// Ram Modules: TagRam, ValidRam, DataRam
//
`include "cache.h"
`include "stdbus.h"

module TagRam (Address, TagIn, TagOut, Write, Clk);

input   [`INDEX]        Address;
input   [`TAG]          TagIn;
output  [`TAG]          TagOut;
input                   Write;
input                   Clk;

reg     [`TAG]  TagOut;
reg     [`TAG]  TagRam[0:`CACHESIZE-1];

always @ (negedge Clk)          // Write
   if (Write) begin
//      $display($time, " TagRam> store addr %d  tag %d", Address, TagIn);
      TagRam[Address] = TagIn;
   end

always @ (negedge Clk) begin    // Read
   TagOut = TagRam[Address];
//   $display($time, " TagRam> tag of %d = %d", Address, TagOut);
end
endmodule /* TagRam */


module ValidRam (Address, ValidIn, ValidOut, Write, Reset, Clk);

input   [`INDEX]        Address;
input                   ValidIn;
output                  ValidOut;
input                   Write;
input                   Reset;
input                   Clk;

reg                     ValidOut;
reg    [`CACHESIZE-1:0] ValidBits;

integer i;

always @ (negedge Clk)          // Write or Reset
   if (Write && !Reset) begin           // Write
//      $display($time, " ValidRam> write valid bit (%b) of %d", ValidIn, Address);
      ValidBits[Address] = ValidIn;
   end
   else if (Reset)                      // Reset
      for (i=0; i<`CACHESIZE; i=i+1)
         ValidBits[i] = `ABSENT;

always @ (negedge Clk) begin    // Read
   ValidOut = ValidBits[Address];
//   $display($time, " ValidRam> valid bit of %d = %d", Address, ValidOut);
end

endmodule /* ValidRam */



module DataRam (Address, DataIn, DataOut, Write, Clk);

input   [`INDEX]        Address;
input   [`DATA]         DataIn;
output  [`DATA]         DataOut;
input                   Write;
input                   Clk;

reg     [`DATA]         DataOut;
reg     [`DATA]         DataRam[`CACHESIZE-1:0];

//always @ (Write) begin
//   $display($time, " DataRam>   Write %b", Write);
//end

always @ (negedge Clk) begin
   if (Write) begin             // Write
//      $display($time, " DataRam> write %d to %d", DataIn, Address);
      DataRam[Address] = DataIn;
   end
end

always @ (negedge Clk)          // Read
   DataOut = DataRam[Address];

endmodule /* DataRam */
